// FORS/FORALL/ERR macros are defined in orts3/libs/kernel/src/Global.H

#include "SampleEventHandler.H"
#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"

#include "consts.H"
#include "parser.H"
#include "print_info.H"
#include "memory.H"

using namespace std;

// tile types from core.bp
// fixme: game script reflection would help here
// in the competition we won't use water tiles
// TILE_CLIFF: non-traversable by ground units

enum { TILE_UNKNOWN=0, TILE_WATER=1, TILE_GROUND=2, TILE_CLIFF=3 };

Terminals t;
Memory m;

bool SampleEventHandler::handle_event(const Event& e)
{
  if (e.get_who() == GameStateModule::FROM) {

    if (e.get_what() == GameStateModule::STOP_MSG) {
      state.quit = true; return true;
    }

    if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
      state.quit = state.error = true; return true;
    }

    if (e.get_what() == GameStateModule::VIEW_MSG) {
	//cout << "foo" << endl;
      compute_actions();
      state.gsm->send_actions();

      return true;
    }
  }

  return false;
}



void move_unit_randomly(GameObj* gob, int maxx, int maxy, Random& rand) {
        ScalarPoint t;

	t.x = rand.rand_uint4() % maxx;
	t.y = rand.rand_uint4() % maxy;
	
	Vector<sint4> params;
        params.push_back(t.x);
        params.push_back(t.y);
        params.push_back(*gob->sod.max_speed);

        gob->set_action("move", params);

	//cout << "move " << t.x << " " << t.y << endl;
}


void move_unit_towards(GameObj* gob, GameObj* target) {
	ScalarPoint t;
	t.x = *target->sod.x;
	t.y = *target->sod.y;
	
	Vector<sint4> params;
        params.push_back(t.x);
        params.push_back(t.y);
        params.push_back(*gob->sod.max_speed);

        gob->set_action("move", params);

	//cout << "move2 " << t.x << " " << t.y << endl;
}




void shoot_unit(const Game& game, GameObj* gob, GameObj* target, char* weapon) {
 	Vector<sint4> args;
	if (MARINE == gob->code()) {
        	args.push_back(game.get_cplayer_info().get_id(target));
       	 	gob->component(weapon)->set_action("attack", args);
		//cout << "M shoot " << game.tick() << " " << ScriptObj::get_obj_id(gob) << endl;
	}
	if (TANK == gob->code()) {
		const sintptr* mode = gob->get_int_ptr("mode");
		if (*mode == 0 || *mode == 3) {
	        	args.push_back(game.get_cplayer_info().get_id(target));
       	 		gob->component(weapon)->set_action("attack", args);
			//cout << "T shoot " << game.tick() << " " << ScriptObj::get_obj_id(gob) << endl;
		} else if (*mode == 1 || *mode == 2) {
			args.push_back(*target->sod.x);
			args.push_back(*target->sod.y);
		       	gob->component(weapon)->set_action("attack", args);
			//cout << "S shoot " << game.tick() << " " << ScriptObj::get_obj_id(gob) << endl;
		}				 
	}
}


void move_unit(GameObj* gob, Tuple& tp) {

	sint4 x = MAX(1, MIN( (sint4)(tp.x), MAXX-1 ) );
	sint4 y = MAX(1, MIN( (sint4)(tp.y), MAXY-1 ) );

	if (TANK == gob->code()) {
		if ( (x == *gob->sod.x) && (y == *gob->sod.y)) {
			if (0 == *(gob->get_int_ptr("mode"))) {
				Vector<sint4> params;
				gob->set_action("switch", params);
				//cout << "switch 0->2" << endl;
				return;
			}
		} else {
			if (2 == *(gob->get_int_ptr("mode"))) {
				Vector<sint4> params;
				gob->set_action("switch", params);
				//cout << "switch 2->0" << endl;
				return;
			}
		}
	}

	Vector<sint4> params;

	params.push_back(x);
        params.push_back(y);
        params.push_back(*gob->sod.max_speed);

        gob->set_action("move", params);

	//cout << "move X = (" << *gob->sod.x << ", " << *gob->sod.y << ") -> (" << x << ", " << y << ")" << endl;

}

// asumming gob1 and gob2 are circle units
sint4 get_distance2(GameObj* gob1, GameObj* gob2) {
	return (*gob1->sod.x - *gob2->sod.x)*(*gob1->sod.x - *gob2->sod.x) + (*gob1->sod.y - *gob2->sod.y)*(*gob1->sod.y - *gob2->sod.y);
}


GameObj* choose_target(GameObj* gob, Terminals& t) {
	sint4 d1 = 1000000;
	sint4 d2 = 1000000;
	
	if (NULL != t.nearest_heavy_enemy) {
		d1 = get_distance2(gob, t.nearest_heavy_enemy);
	}

	if (NULL != t.nearest_light_enemy) {
		d2 = get_distance2(gob, t.nearest_light_enemy);
	}
	
	if (d1 < d2)
		return t.nearest_heavy_enemy;
	else 
		return t.nearest_light_enemy;
}


sint4 get_weapon_max_range(GameObj* gob, char* weapon_name) {
	ScriptObj * weapon = gob->component(weapon_name);
	if (!weapon) return -1;

	sintptr * prange = weapon->get_int_ptr("max_ground_range");
	if (!prange) return -1;
	return *prange * *prange;
}


void SampleEventHandler::compute_actions()
{
	const Game &game = state.gsm->get_game();
	const sint4 cid = game.get_client_player();
	
	const Game::ObjCont &objs = game.get_objs(cid);
	
	memory_init(game, m);
	
	//cout << "LIVE " << game.tick() << endl;
	
	FORALL (objs, it) {

		GameObj * gob = (*it)->get_GameObj();

		if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {

			update_terminals(game, gob, t);
			update_terminals2(game, t, m);

	//		cout << "pro" << endl;	
		//	show_terminals(t);
	//		cout << "po" << endl;
	//
			Tuple* tpPtr;
			
		//	cout << strategy << endl;

			if (0 == strategy) {
				if (TANK == gob->code()) {
					tpPtr = &(IF(ADD(OPP(t.B[5]),ADD(MUL(ADD(LT(t.R[9],LEN(t.V[19])),OPP(t.B[0])),LT(MAX(OPP(t.R[20]),SGN(OPP(t.R[2]))),MUL(ADD(t.R[7],ABS(IF(ADD(LT(SUB(LEN(IF(ADD(MUL(t.B[2],LT(MUL(IF(OPP(OPP(ADD(ADD(OPP(t.B[5]),t.B[2]),t.B[2]))),t.R[22],t.R[20]),0.12104750248493668),t.R[12])),t.B[8]),t.V[4],t.V[16])),t.R[22]),t.R[5]),t.B[6]),t.R[1],t.R[18]))),t.R[17]))),t.B[6])),t.V[13],t.V[22]));
				} else {
					tpPtr = &(IF(t.B[4],IF(t.B[7],LEF(OPP(t.V[18])),t.V[13]),LEF(t.V[18])));
				}
			} else if (strategy == 1) {
				if (TANK == gob->code()) {
					tpPtr = &(t.V[2]);
				} else {
					tpPtr = &(IF(t.B[11],IF(t.B[7],t.V[18],t.V[19]),t.V[17]));
				}
			} else {
				if (TANK == gob->code()) {
					tpPtr = &(t.V[18]);
				} else {
					tpPtr = &(t.V[19]);
				}
	
			}

			Tuple& tp = *tpPtr;
			
			tp = convert_vector_to_map(gob, t, tp);

			move_unit(gob, tp);
			Unit* u = find_mate(gob, m);

			if ( (u->last_tick_shoot==-1) || (u->last_tick_shoot + COOLDOWN <= game.tick()) ) {
				GameObj* target = choose_target(gob, t);
				shoot_unit(game, gob, target, t.curr_weapon);
				remember_unit(gob, u, tp.x, tp.y, game.tick()); 
			} else {
				remember_unit(gob, u, tp.x, tp.y, -1); 
			}

		}	
	}

	remember_enemies(game, m);
	//show_memory(m);
	
}
