
#include "Global.H"
#include "Options.H"
#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"

#include "SampleEventHandler.H"

using namespace std;

//===================================================================

static char * VERSION = "sampleai v0.1";

//===================================================================

void add_options()
{
  Options opt("SampleClient");
  opt.put("-strategy", (sint4)(0), "which strategy should a client play");
 // opt.put("-cinfo", "display game state changes");
}

//===================================================================

// loop is called

class Looper //: public LoopFunctor
{
public:

  Looper(SampleAIState * state_) : state(state_) {
  }

  void loop() const {
    // looks for server messages
    // if one or more arrived, calls handlers
    // if not, sleeps for one millisecond
    if (!state->gsm->recv_view()) SDL_Delay(1);
  }

private:
  SampleAIState * state;
};

//===================================================================


int main(int argc, char * * argv)
{
  signals_shut_down(true);

  // populate opt with options from various modules

  add_options();
  GameStateModule::Options::add();
  Game::Options::add();

  // process command line arguments
  if (Options::process(argc, argv, cerr, VERSION)) exit(20);

  // initialize SDL
  SDLinit::network_init();

  // populate option objects with command line values
  // and create modules
  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);

  // change seed value to time(0) if 0
  sint4 seed;
  Options::get("-seed", seed);
//  if (!seed)
  	seed = time(0);
  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  // gives event handler access to the state
  SampleAIState state(&gsm);
  SampleEventHandler sev(state, seed, 0);

  // gsm events are handled in sev
  gsm.add_handler(&sev);

  // connect game state module to server
  if (!gsm.connect()) ERR("connection problems");

  Looper l(&state);
  
  //a must-be for ECJ and evaluation do not delete!
  std::cout << "client=" << gsm.get_game().get_client_player() << std::endl;

  //doit
  while (!state.quit) l.loop();
  
  return 0;
}

