#include <math.h>
#include <string.h>

#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "ServerObjData.H"

#include "consts.H"
#include "terminals.H"
#include "parser.H"
#include "memory.H"

using namespace std;

void show_unit(Unit& u) {
	cout << u.gob_id << " " << u.x << " " << u.y << " " << u.hp << u.last_tick_shoot << endl;
}

void show_memory(Memory& m) {
	cout << "MATES" << endl;
	for (int i = 0; i < MAX_UNITS; ++i) 
		show_unit(m.mate[i]);
		
	cout << "ENEMIES" << endl;
	for (int i = 0; i < MAX_UNITS; ++i) 
		show_unit(m.enemy[i]);

}


Unit* find_enemy(GameObj* gob, Memory& m) {
	for (int i = 0; i < MAX_UNITS; ++i) {
		if ((int)(ScriptObj::get_obj_id(gob)) == m.enemy[i].gob_id) {
			return &(m.enemy[i]);
		}
	}
	return NULL;
}

Unit* find_mate(GameObj* gob, Memory& m) {
	for (int i = 0; i < MAX_UNITS; ++i) {
		if ((int)(ScriptObj::get_obj_id(gob)) == m.mate[i].gob_id) {
			return &(m.mate[i]);
		}
	}
	return NULL;
}

void remember_unit(GameObj* gob, Unit* u, float x, float y, int last_tick_shoot) {
	u->gob_id = ScriptObj::get_obj_id(gob);
	u->x = *gob->sod.x;
	u->y = *gob->sod.y;
	if ((x != -1) && (y != -1)) {
		u->x2 = MAX(1, MIN( (sint4)(x), MAXX-1 ) );
		u->y2 = MAX(1, MIN( (sint4)(y), MAXY-1 ) );
	}
	u->hp = *gob->get_int_ptr("hp");
	if (-1 != last_tick_shoot) {
		u->last_tick_shoot = last_tick_shoot;
	}
}






void remember_enemies(const Game& game, Memory& m) {
	const sint4 id = game.get_client_player();
	const Game::ObjCont& objs = game.get_objs((id+1)%2);

	
	FORALL(objs, it) {
		GameObj * enemy = (*it)->get_GameObj();
					
		if (enemy && enemy->sod.in_game && *enemy->sod.x >= 0 && !enemy->is_dead()) {
		
			Unit* u = find_enemy(enemy, m);
			if (NULL != u) {
				remember_unit(enemy, u, -1, -1, -1);
			}
		

		}
	}
	
	
}




void memory_init(const Game& game, Memory& m) {

	if (true == m.init) return;

	//init enemies
	const sint4 id = game.get_client_player();

	{
	const Game::ObjCont& objs = game.get_objs((id+1)%2);
	int i = 0;
	FORALL(objs, it) {
		GameObj * enemy = (*it)->get_GameObj();			
		if (enemy && enemy->sod.in_game && *enemy->sod.x >= 0 && !enemy->is_dead()) {
			remember_unit(enemy, &(m.enemy[i]), -1, -1, -1);
			++i;
		}
	}
	}
	
	//init mates
	{
	const Game::ObjCont& objs = game.get_objs(id);
	int i = 0;
	FORALL(objs, it) {
		GameObj * mate = (*it)->get_GameObj();			
		if (mate && mate->sod.in_game && *mate->sod.x >= 0 && !mate->is_dead()) {
			remember_unit(mate, &(m.mate[i]), -1, -1, -1);
			++i;
		}
	}
	}
	
	m.init = true;

}

