#ifndef __PRINT_INFO_H
#define __PRINT_INFO_H


#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"

#include "print_info.H"
#include "consts.H"
#include <string.h>

#define P(x) ((x) != NULL ? *(x) : 0)

using namespace std;

// name obj_id code hp max_hp curr_action_id
void print_generic_info(GameObj* gob) {
	const sintptr *hp = gob->get_int_ptr("hp"); 
	const sintptr *max_hp = gob->get_int_ptr("max_hp");
	cout << gob->bp_name() << " " << ScriptObj::get_obj_id(gob) << " " << gob->code() << " " << P(hp) << " " << P(max_hp) << " " << gob->cur_action.get_id() << endl;
}


// x, y, size, speed, max_speed, sight, is_moving
void print_moving_info( GameObj* gob) {
	const ServerObjData &sod = gob->sod;

	if (*sod.shape == Object::CIRCLE) {
		cout << "(" << *sod.x << ", " << *sod.y << ", " << *sod.radius << ")";
	} else if (*sod.shape == Object::RECTANGLE) {
        	cout << "(" << *sod.x1 << ", " << *sod.y1 << ", " << *sod.x2 << ", " << *sod.y2 << ")";
      	} else {
        	cout << "(?)";
      	}
	
	cout << " " << *sod.speed << " " << *sod.max_speed << " " << *sod.sight << " " << *sod.is_moving << endl;
}


// min_range, max_range, min_damage, max_damage, cooldown
void print_weapon_info(ScriptObj* weapon) {

	if (NULL == weapon) return;

	sintptr * min_range  = weapon->get_int_ptr("min_ground_range");
	sintptr * max_range  = weapon->get_int_ptr("max_ground_range");
	sintptr * max_damage = weapon->get_int_ptr("max_damage");
	sintptr * min_damage = weapon->get_int_ptr("min_damage");
	sintptr * cooldown   = weapon->get_int_ptr("cooldown");

	cout << P(min_range) << " " << P(max_range) << " " << P(min_damage) << " " << P(max_damage) << " " << P(cooldown) << endl;
}


void print_marine_info(GameObj* gob) {
	cout << "weapon ";
	print_weapon_info( gob->component("weapon") );
}


void print_tank_info(GameObj* gob) {
	cout << "weapon ";
	print_weapon_info( gob->component("weapon") );
	cout << "weapon2 ";
	print_weapon_info( gob->component("weapon2") );

	const sintptr* mode = gob->get_int_ptr("mode");
	cout << "mode is " << *mode ;
	switch (*mode) {
		case 0:	cout << " weapon"; break;
		case 1: cout << " weapon -> weapon2"; break;
		case 2: cout << " weapon2"; break;
		case 3: cout << " weapon2 -> weapon"; break;
	}
		
	const sintptr* switch_time = gob->get_int_ptr("switch_time");
	const sintptr* switch_pos = gob->get_int_ptr("switch_pos");
	
	cout << " " << *switch_pos << " " << *switch_time << endl;
}


void print_unit_info(GameObj* gob) {
	if (MARINE == gob->code()) {
		print_marine_info(gob);
	} else if (TANK == gob->code()) {
		print_tank_info(gob);
	}
}



void print_gob_info(GameObj* gob) {
      
	//object in game
	if ( !(gob->sod.in_game) ) { 
 		cout << " no gameobj" << endl; return; 
	}
	
	//object on the playfield
	if ( *(gob->sod.x) < 0 ) { 
		cout << "not on playfield" << endl; return; 
	}

	print_generic_info(gob);
	print_moving_info(gob);
	print_unit_info(gob);

	cout << endl;
}


#endif
