#include <math.h>
#include <string.h>

#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "ServerObjData.H"

#include "consts.H"
#include "terminals.H"
#include "parser.H"
#include "memory.H"

using namespace std;

void show_terminals(Terminals& t) {
	for (int i = 0; i < R_SIZE; ++i) {
		cout << i << "=" << t.R[i] << " ";
	}
	cout << endl;
	for (int i = 0; i < V_SIZE; ++i) {
		cout << i << "=" << t.V[i].x << "|" << t.V[i].y << " ";
	}
	cout << endl;
	for (int i = 0; i < B_SIZE; ++i) {
		cout << i << "=" << t.B[i] << " ";
	}
	cout << endl << endl;

}

void reset_vectors(Terminals& t) {
	for (int i = 0; i < V_SIZE; ++i) {
		t.V[i].x = 0;
		t.V[i].y = 0;
	}
}

float get_distance(GameObj* gob1, GameObj* gob2) {
	return sqrt((*gob1->sod.x - *gob2->sod.x)*(*gob1->sod.x - *gob2->sod.x) + (*gob1->sod.y - *gob2->sod.y)*(*gob1->sod.y - *gob2->sod.y));
}


void set_global(const Game& game, Terminals& t) {

	//timing
	t.R[R_TICK] = ( game.tick() + 0.0 ) / (MAX_TICKS + 0.0);
	
	if (t.R[R_TICK] < 0.25) {
		t.B[B_IS_BEGIN_TIME] = true;
	} else {	
		t.B[B_IS_BEGIN_TIME] = false;
	}

}

int get_range(GameObj* gob) {
	if (MARINE == gob->code()) {
		return MARINE_RANGE;
	} else if (TANK == gob->code()) {
		const sintptr* mode = gob->get_int_ptr("mode");
		if (*mode == 0 || *mode == 3) {
			return TANK_RANGE1;
		} else if (*mode == 1 || *mode == 2) {
			return TANK_RANGE2;
		}
	}
	return 0;
}

void set_me(GameObj* gob, Terminals& t) {

	t.me = gob;
	t.code = gob->code();
	int x = *gob->sod.x;
	int y = *gob->sod.y;


	//set me 0,0 vector
	t.V[V_ME].x = 0;
	t.V[V_ME].y = 0;

	//set my home
	if ( (t.home_mine.x == -1) && (t.home_mine.y == -1) ) {
		t.home_mine.x = x;
		t.home_mine.x = y;
	}

	t.V[V_HOME_MINE].x = (t.home_mine.x - (x+0.0)) / (MAXX+0.0);
	t.V[V_HOME_MINE].y = (t.home_mine.y - (y+0.0)) / (MAXY+0.0);

	// set my energy
	if (MARINE == t.code) {
		t.max_speed = MARINE_SPEED;
		t.R[R_MY_ENERGY] = (*gob->get_int_ptr("hp")+0.0) / (START_MARINE_HP+0.0);
	} else if (TANK == t.code) {
		t.max_speed = TANK_SPEED;
		t.R[R_MY_ENERGY] = (*gob->get_int_ptr("hp")+0.0) / (START_TANK_HP+0.0);
	}

	// set am dying and healthy
	t.B[B_AM_DYING] = false;
	if (t.R[R_MY_ENERGY] < 0.2) {
		t.B[B_AM_DYING] = true;
	}
	t.B[B_AM_HEALTHY] = false;
	if (t.R[R_MY_ENERGY] > 0.8) {
		t.B[B_AM_HEALTHY] = true;
	}


	// set weapon, needed later
	if (MARINE == t.code) {
		t.range = MARINE_RANGE;
		strncpy(t.curr_weapon, "weapon", WEAPON_BUFFER);
	} else if (TANK == t.code) {
		const sintptr* mode = gob->get_int_ptr("mode");
		if (*mode == 0 || *mode == 3) {
			t.range = TANK_RANGE1;
			strncpy(t.curr_weapon, "weapon", WEAPON_BUFFER);
		} else if (*mode == 1 || *mode == 2) {
			t.range = TANK_RANGE2;
			strncpy(t.curr_weapon, "weapon2", WEAPON_BUFFER);
		}
	
	}

	//set am mobile
	t.B[B_AM_MOBILE] = true;
	if (TANK == t.code) {
		const sintptr* mode = gob->get_int_ptr("mode");
		if (*mode == 1 || *mode == 2) {
			t.B[B_AM_MOBILE] = false;
		}
	}

	// set am on edge
	if ( (x < 0.03 * (MAXX+0.0)) || (x > 0.97 * (MAXX+0.0)) || (y < 0.04 * (MAXY+0.0)) || (y > 0.96 * (MAXY+0.0)) ) {
		t.B[B_AM_ON_EDGE] = true;
	} else {
		t.B[B_AM_ON_EDGE] = false;
	}

	
}


#define TMP_MAX 10000000


void iterate_over_mates(const Game& game, GameObj* gob, Terminals& t) {
	const sint4 id = game.get_client_player();
	const Game::ObjCont& objs = game.get_objs(id);
	
	t.all_light_mate_num = 0;
	t.all_heavy_mate_num = 0;
	t.range_any_mate_num = 0;
	t.range_light_mate_num = 0;
	t.range_heavy_mate_num = 0;

	t.center_all_mate.x = 0;
	t.center_light_mate.x = 0;
	t.center_heavy_mate.x = 0;
	t.center_all_mate.y = 0;
	t.center_light_mate.y = 0;
	t.center_heavy_mate.y = 0;

	t.nearest_light_mate = NULL;
	t.furthest_range_light_mate = NULL;
	t.weakest_myrange_light_mate = NULL;
	t.nearest_heavy_mate = NULL;
	t.furthest_range_heavy_mate = NULL;
	t.weakest_myrange_heavy_mate = NULL;

	t.hp_team = 0;
	
	float minLightDist = TMP_MAX;
	float maxRangeLightDist = 0;
	float maxAllLightDist = 0;
	float minHeavyDist = TMP_MAX;
	float maxRangeHeavyDist = 0;
	float maxAllHeavyDist = 0;
	int minRangeLightHP = TMP_MAX;
	int minAllLightHP = TMP_MAX;
	int minRangeHeavyHP = TMP_MAX;
	int minAllHeavyHP = TMP_MAX;
	
	FORALL(objs, it) {
		GameObj * mate = (*it)->get_GameObj();
					
		if (mate && mate->sod.in_game && *mate->sod.x >= 0 && !mate->is_dead()) {
		
			if (gob == mate)
				continue;
		
			sint4 mate_code = mate->code();
			sint4 range = get_range(mate);			
			float dist = get_distance(gob, mate);
			sint4 x = *mate->sod.x;
			sint4 y = *mate->sod.y;
			sint4 hp = *mate->get_int_ptr("hp");

			//update team hp
			t.hp_team += hp;
			
			// set all center
			t.center_all_mate.x += x;
			t.center_all_mate.y += y;

			if (MARINE == mate_code) {
						
				// set center light
				t.center_light_mate.x += x;
				t.center_light_mate.y += y;
				
				// set all light num
				t.all_light_mate_num++;		
				
				//set all nearest light
				if (dist < minLightDist) {
					minLightDist = dist;
					t.nearest_light_mate = mate;
				}		
				
				// set fallback all furthest light
				if (maxRangeLightDist == 0) {
					if (dist > maxAllLightDist) {
						maxAllLightDist = dist;
						t.furthest_range_light_mate = mate;
					}
				}
				
				// set range
				if (dist < range) {
				
					//set main range furthest light
					if (dist > maxRangeLightDist) {
						maxRangeLightDist = dist;
						t.furthest_range_light_mate = mate;
					}

					//set range any num
					t.range_any_mate_num++;

					//set range light num
					t.range_light_mate_num++;
				}			   
				
				
				if ( (TANK_RANGE2 == t.range) && (dist < TANK_RANGE0) ) {
					continue;
				}
	
				//set fallback all weakest light
				if (minRangeLightHP == TMP_MAX) {
					if (hp < minAllLightHP) {
						minAllLightHP = hp;
						t.weakest_myrange_light_mate = mate;
					}
				}
	
				//set myrange 
				if (dist < t.range) {
				
					//set main range weakest light
					if (hp < minRangeLightHP) {
						minRangeLightHP = hp;
						t.weakest_myrange_light_mate = mate;
					}
				}
				
			} else if (TANK == mate_code) {
			
				// set center heavy
				t.center_heavy_mate.x += x;
				t.center_heavy_mate.y += y;

				// set all heavy num
				t.all_heavy_mate_num++;

				//set all nearest heavy
				if (dist < minHeavyDist) {
					minHeavyDist = dist;
					t.nearest_heavy_mate = mate;
				}

				//set fallback all furhtest heavy
				if (maxRangeHeavyDist == 0) {
					if (dist > maxAllHeavyDist) {
						maxAllHeavyDist = dist;
						t.furthest_range_heavy_mate = mate;
					}
				}
	
				//set range
				if (dist < range) {

					//set main range furthest heavy
					if (dist > maxRangeHeavyDist) {
						maxRangeHeavyDist = dist;
						t.furthest_range_heavy_mate = mate;
					}

					//set range any num
					t.range_any_mate_num++;

					//set range heavy num
					t.range_heavy_mate_num++;
				}				


				if ( (TANK_RANGE2 == t.range) && (dist < TANK_RANGE0) ) {
					continue;
				}
	

				//set fallback all weakest heavy
				if (minRangeHeavyHP == TMP_MAX) {
					if (hp < minAllHeavyHP) {
						minAllHeavyHP = hp;
						t.weakest_myrange_heavy_mate = mate;
					}
				}
	
				//set myrange 
				if (dist < t.range) {
			
					//set main range weakest heavy
					if (hp < minRangeHeavyHP) {
						minRangeHeavyHP = hp;
						t.weakest_myrange_heavy_mate = mate;
					}
				}
	
			}
		}
	}
	
	int x = *gob->sod.x;
	int y = *gob->sod.y;

	if (NULL != t.nearest_light_mate) {
		t.R[R_NEAREST_LIGHT_MATE_ENERGY] = (*(t.nearest_light_mate->get_int_ptr("hp"))+0.0)/(MARINE_HP+0.0);
		t.V[V_NEAREST_LIGHT_MATE].x = (*(t.nearest_light_mate->sod.x) - x) / (MAXX+0.0);
		t.V[V_NEAREST_LIGHT_MATE].y = (*(t.nearest_light_mate->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.nearest_heavy_mate) {
		t.R[R_NEAREST_HEAVY_MATE_ENERGY] = (*(t.nearest_heavy_mate->get_int_ptr("hp"))+0.0)/(TANK_HP+0.0);
		t.V[V_NEAREST_HEAVY_MATE].x = (*(t.nearest_heavy_mate->sod.x) - x) / (MAXX+0.0);
		t.V[V_NEAREST_HEAVY_MATE].y = (*(t.nearest_heavy_mate->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.weakest_myrange_light_mate) {
		t.R[R_WEAKEST_MYRANGE_LIGHT_MATE_ENERGY] = (*(t.weakest_myrange_light_mate->get_int_ptr("hp"))+0.0)/(MARINE_HP+0.0);
		t.V[V_WEAKEST_MYRANGE_LIGHT_MATE].x = (*(t.weakest_myrange_light_mate->sod.x) - x) / (MAXX+0.0);
		t.V[V_WEAKEST_MYRANGE_LIGHT_MATE].y = (*(t.weakest_myrange_light_mate->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.weakest_myrange_heavy_mate) {
		t.R[R_WEAKEST_MYRANGE_HEAVY_MATE_ENERGY] = (*(t.weakest_myrange_heavy_mate->get_int_ptr("hp"))+0.0)/(TANK_HP+0.0);
		t.V[V_WEAKEST_MYRANGE_HEAVY_MATE].x = (*(t.weakest_myrange_heavy_mate->sod.x) - x) / (MAXX+0.0);
		t.V[V_WEAKEST_MYRANGE_HEAVY_MATE].y = (*(t.weakest_myrange_heavy_mate->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.furthest_range_light_mate) {
		t.V[V_FURTHEST_RANGE_LIGHT_MATE].x = (*(t.furthest_range_light_mate->sod.x) - x) / (MAXX+0.0);
		t.V[V_FURTHEST_RANGE_LIGHT_MATE].y = (*(t.furthest_range_light_mate->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.furthest_range_heavy_mate) {
		t.V[V_FURTHEST_RANGE_HEAVY_MATE].x = (*(t.furthest_range_heavy_mate->sod.x) - x) / (MAXX+0.0);
		t.V[V_FURTHEST_RANGE_HEAVY_MATE].y = (*(t.furthest_range_heavy_mate->sod.y) - y) / (MAXY+0.0);
	}
	

	t.R[R_RANGE_ANY_MATE_NUM] = t.range_any_mate_num / (START_N+0.0);
	t.R[R_RANGE_LIGHT_MATE_NUM] = t.range_light_mate_num / (START_MARINE_N+0.0);
	t.R[R_RANGE_HEAVY_MATE_NUM] = t.range_heavy_mate_num / (START_TANK_N+0.0);
	t.R[R_ALL_LIGHT_MATE_NUM] = t.all_light_mate_num / (START_MARINE_N+0.0);
	t.R[R_ALL_HEAVY_MATE_NUM] = t.all_heavy_mate_num / (START_TANK_N+0.0);

	if (0 != t.all_light_mate_num + t.all_heavy_mate_num) {
		t.V[V_CENTER_ALL_MATE].x = t.center_all_mate.x / (t.all_light_mate_num + t.all_heavy_mate_num + 0.0);
		t.V[V_CENTER_ALL_MATE].y = t.center_all_mate.y / (t.all_light_mate_num + t.all_heavy_mate_num + 0.0);
	}
	
	if ( (t.home_team.x == -1) && (t.home_team.y == -1) ) {
		t.home_team.x = t.V[V_CENTER_ALL_MATE].x;
		t.home_team.y = t.V[V_CENTER_ALL_MATE].y;
	}

	if (0 != t.all_light_mate_num) {
		t.V[V_CENTER_LIGHT_MATE].x = t.center_light_mate.x / (t.all_light_mate_num + 0.0);
		t.V[V_CENTER_LIGHT_MATE].y = t.center_light_mate.y / (t.all_light_mate_num + 0.0);
	}

	if (0 != t.all_heavy_mate_num) {
		t.V[V_CENTER_HEAVY_MATE].x = t.center_heavy_mate.x / (t.all_heavy_mate_num + 0.0);
		t.V[V_CENTER_HEAVY_MATE].y = t.center_heavy_mate.y / (t.all_heavy_mate_num + 0.0);
	}

	t.V[V_CENTER_ALL_MATE].x = (t.V[V_CENTER_ALL_MATE].x - (x + 0.0)) / (MAXX+0.0);
	t.V[V_CENTER_ALL_MATE].y = (t.V[V_CENTER_ALL_MATE].y - (y + 0.0)) / (MAXY+0.0);

	t.V[V_CENTER_LIGHT_MATE].x = (t.V[V_CENTER_LIGHT_MATE].x - (x + 0.0)) / (MAXX+0.0);
	t.V[V_CENTER_LIGHT_MATE].y = (t.V[V_CENTER_LIGHT_MATE].y - (y + 0.0)) / (MAXY+0.0);

	t.V[V_CENTER_HEAVY_MATE].x = (t.V[V_CENTER_HEAVY_MATE].x - (x + 0.0)) / (MAXX+0.0);
	t.V[V_CENTER_HEAVY_MATE].y = (t.V[V_CENTER_HEAVY_MATE].y - (y + 0.0)) / (MAXY+0.0);

	t.V[V_HOME_TEAM].x = (t.home_team.x - (x+0.0)) / (MAXX+0.0); 
	t.V[V_HOME_TEAM].y = (t.home_team.y - (y+0.0)) / (MAXY+0.0); 

	
}



void iterate_over_enemies(const Game& game, GameObj* gob, Terminals& t) {
	const sint4 id = game.get_client_player();
	const Game::ObjCont& objs = game.get_objs((id+1)%2);

	t.myrange_any_enemy_num = 0;
	t.myrange_light_enemy_num = 0;
	t.myrange_heavy_enemy_num = 0;
	t.all_light_enemy_num = 0;
	t.all_heavy_enemy_num = 0;
	t.range_any_enemy_num = 0;
	t.range_light_enemy_num = 0;
	t.range_heavy_enemy_num = 0;

	t.center_all_enemy.x = 0;
	t.center_light_enemy.x = 0;
	t.center_heavy_enemy.x = 0;
	t.center_all_enemy.y = 0;
	t.center_light_enemy.y = 0;
	t.center_heavy_enemy.y = 0;

	t.hp_enemy = 0;

	t.nearest_light_enemy = NULL;
	t.furthest_range_light_enemy = NULL;
	t.weakest_myrange_light_enemy = NULL;
	t.nearest_heavy_enemy = NULL;
	t.furthest_range_heavy_enemy = NULL;
	t.weakest_myrange_heavy_enemy = NULL;
	
	float minLightDist = TMP_MAX;
	float maxRangeLightDist = 0;
	float maxAllLightDist = 0;
	float minHeavyDist = TMP_MAX;
	float maxRangeHeavyDist = 0;
	float maxAllHeavyDist = 0;
	int minRangeLightHP = TMP_MAX;
	int minAllLightHP = TMP_MAX;
	int minRangeHeavyHP = TMP_MAX;
	int minAllHeavyHP = TMP_MAX;
	
	FORALL(objs, it) {
		GameObj * enemy = (*it)->get_GameObj();
					
		if (enemy && enemy->sod.in_game && *enemy->sod.x >= 0 && !enemy->is_dead()) {
		
			if (gob == enemy)
				continue;
		
			sint4 enemy_code = enemy->code();
			sint4 range = get_range(enemy);			
			float dist = get_distance(gob, enemy);
			sint4 x = *enemy->sod.x;
			sint4 y = *enemy->sod.y;
			sint4 hp = *enemy->get_int_ptr("hp");

			//update hp enemy
			t.hp_enemy += hp;
			
			// set all center
			t.center_all_enemy.x += x;
			t.center_all_enemy.y += y;

			if (MARINE == enemy_code) {
						
				// set center light
				t.center_light_enemy.x += x;
				t.center_light_enemy.y += y;
				
				// set all light num
				t.all_light_enemy_num++;		
				
				//set all nearest light
				if (dist < minLightDist) {
					minLightDist = dist;
					t.nearest_light_enemy = enemy;
				}		
				
				// set fallback all furthest light
				if (maxRangeLightDist == 0) {
					if (dist > maxAllLightDist) {
						maxAllLightDist = dist;
						t.furthest_range_light_enemy = enemy;
					}
				}
				
				// set range
				if (dist < range) {
				
					//set main range furthest light
					if (dist > maxRangeLightDist) {
						maxRangeLightDist = dist;
						t.furthest_range_light_enemy = enemy;
					}

					//set range any num
					t.range_any_enemy_num++;

					//set range light num
					t.range_light_enemy_num++;
				}			   
				
				if ( (TANK_RANGE2 == t.range) && (dist < TANK_RANGE0) ) {
					continue;
				}
	
				//set fallback all weakest light
				if (minRangeLightHP == TMP_MAX) {
					if (hp < minAllLightHP) {
						minAllLightHP = hp;
						t.weakest_myrange_light_enemy = enemy;
					}
				}
	
				//set myrange 
				if (dist < t.range) {
									
					//set myrange any num
					t.myrange_any_enemy_num++;

					//set myrange light num
					t.myrange_light_enemy_num++;
					
					//set main range weakest light
					if (hp < minRangeLightHP) {
						minRangeLightHP = hp;
						t.weakest_myrange_light_enemy = enemy;
					}
				}
				
			} else if (TANK == enemy_code) {
			
				// set center heavy
				t.center_heavy_enemy.x += x;
				t.center_heavy_enemy.y += y;

				// set all heavy num
				t.all_heavy_enemy_num++;

				//set all nearest heavy
				if (dist < minHeavyDist) {
					minHeavyDist = dist;
					t.nearest_heavy_enemy = enemy;
				}

				//set fallback all furhtest heavy
				if (maxRangeHeavyDist == 0) {
					if (dist > maxAllHeavyDist) {
						maxAllHeavyDist = dist;
						t.furthest_range_heavy_enemy = enemy;
					}
				}
	
				//set range
				if (dist < range) {

					//set main range furthest heavy
					if (dist > maxRangeHeavyDist) {
						maxRangeHeavyDist = dist;
						t.furthest_range_heavy_enemy = enemy;
					}

					//set range any num
					t.range_any_enemy_num++;

					//set range heavy num
					t.range_heavy_enemy_num++;
				}				


				if ( (TANK_RANGE2 == t.range) && (dist < TANK_RANGE0) ) {
					continue;
				}
				

				//set fallback all weakest heavy
				if (minRangeHeavyHP == TMP_MAX) {
					if (hp < minAllHeavyHP) {
						minAllHeavyHP = hp;
						t.weakest_myrange_heavy_enemy = enemy;
					}
				}
	
				//set myrange 
				if (dist < t.range) {
					
					//set myrange any num
					t.myrange_any_enemy_num++;

					//set myrange heavy num
					t.myrange_heavy_enemy_num++;
					
					//set main range weakest heavy
					if (hp < minRangeHeavyHP) {
						minRangeHeavyHP = hp;
						t.weakest_myrange_heavy_enemy = enemy;
					}
				}
	
			}
		}
	}
	
	int x = *gob->sod.x;
	int y = *gob->sod.y;

	if (NULL != t.nearest_light_enemy) {
		t.R[R_NEAREST_LIGHT_ENEMY_ENERGY] = (*(t.nearest_light_enemy->get_int_ptr("hp"))+0.0)/(MARINE_HP+0.0);
		t.V[V_NEAREST_LIGHT_ENEMY].x = (*(t.nearest_light_enemy->sod.x) - x) / (MAXX+0.0);
		t.V[V_NEAREST_LIGHT_ENEMY].y = (*(t.nearest_light_enemy->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.nearest_heavy_enemy) {
		t.R[R_NEAREST_HEAVY_ENEMY_ENERGY] = (*(t.nearest_heavy_enemy->get_int_ptr("hp"))+0.0)/(TANK_HP+0.0);
		t.V[V_NEAREST_HEAVY_ENEMY].x = (*(t.nearest_heavy_enemy->sod.x) - x) / (MAXX+0.0);
		t.V[V_NEAREST_HEAVY_ENEMY].y = (*(t.nearest_heavy_enemy->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.weakest_myrange_light_enemy) {
		t.R[R_WEAKEST_MYRANGE_LIGHT_ENEMY_ENERGY] = (*(t.weakest_myrange_light_enemy->get_int_ptr("hp"))+0.0)/(MARINE_HP+0.0);
		t.V[V_WEAKEST_MYRANGE_LIGHT_ENEMY].x = (*(t.weakest_myrange_light_enemy->sod.x) - x) / (MAXX+0.0);
		t.V[V_WEAKEST_MYRANGE_LIGHT_ENEMY].y = (*(t.weakest_myrange_light_enemy->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.weakest_myrange_heavy_enemy) {
		t.R[R_WEAKEST_MYRANGE_HEAVY_ENEMY_ENERGY] = (*(t.weakest_myrange_heavy_enemy->get_int_ptr("hp"))+0.0)/(TANK_HP+0.0);
		t.V[V_WEAKEST_MYRANGE_HEAVY_ENEMY].x = (*(t.weakest_myrange_heavy_enemy->sod.x) - x) / (MAXX+0.0);
		t.V[V_WEAKEST_MYRANGE_HEAVY_ENEMY].y = (*(t.weakest_myrange_heavy_enemy->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.furthest_range_light_enemy) {
		t.V[V_FURTHEST_RANGE_LIGHT_ENEMY].x = (*(t.furthest_range_light_enemy->sod.x) - x) / (MAXX+0.0);
		t.V[V_FURTHEST_RANGE_LIGHT_ENEMY].y = (*(t.furthest_range_light_enemy->sod.y) - y) / (MAXY+0.0);
	}

	if (NULL != t.furthest_range_heavy_enemy) {
		t.V[V_FURTHEST_RANGE_HEAVY_ENEMY].x = (*(t.furthest_range_heavy_enemy->sod.x) - x) / (MAXX+0.0);
		t.V[V_FURTHEST_RANGE_HEAVY_ENEMY].y = (*(t.furthest_range_heavy_enemy->sod.y) - y) / (MAXY+0.0);
	}
	
	t.R[R_MYRANGE_ANY_ENEMY_NUM] = t.myrange_any_enemy_num / (START_N+0.0);
	t.R[R_MYRANGE_LIGHT_ENEMY_NUM] = t.myrange_light_enemy_num / (START_MARINE_N+0.0);
	t.R[R_MYRANGE_HEAVY_ENEMY_NUM] = t.myrange_heavy_enemy_num / (START_TANK_N+0.0);
	t.R[R_RANGE_ANY_ENEMY_NUM] = t.range_any_enemy_num / (START_N+0.0);
	t.R[R_RANGE_LIGHT_ENEMY_NUM] = t.range_light_enemy_num / (START_MARINE_N+0.0);
	t.R[R_RANGE_HEAVY_ENEMY_NUM] = t.range_heavy_enemy_num / (START_TANK_N+0.0);
	t.R[R_ALL_LIGHT_ENEMY_NUM] = t.all_light_enemy_num / (START_MARINE_N+0.0);
	t.R[R_ALL_HEAVY_ENEMY_NUM] = t.all_heavy_enemy_num / (START_TANK_N+0.0);

	if (0 != t.all_light_enemy_num + t.all_heavy_enemy_num) {
		t.V[V_CENTER_ALL_ENEMY].x = t.center_all_enemy.x / (t.all_light_enemy_num + t.all_heavy_enemy_num + 0.0);
		t.V[V_CENTER_ALL_ENEMY].y = t.center_all_enemy.y / (t.all_light_enemy_num + t.all_heavy_enemy_num + 0.0);
	}
	
	if (0 != t.all_light_enemy_num) {
		t.V[V_CENTER_LIGHT_ENEMY].x = t.center_light_enemy.x / (t.all_light_enemy_num + 0.0);
		t.V[V_CENTER_LIGHT_ENEMY].y = t.center_light_enemy.y / (t.all_light_enemy_num + 0.0);
	}
	
	if (0 != t.all_heavy_enemy_num) {
		t.V[V_CENTER_HEAVY_ENEMY].x = t.center_heavy_enemy.x / (t.all_heavy_enemy_num + 0.0);
		t.V[V_CENTER_HEAVY_ENEMY].y = t.center_heavy_enemy.y / (t.all_heavy_enemy_num + 0.0);
	}

	t.V[V_CENTER_ALL_ENEMY].x = (t.V[V_CENTER_ALL_ENEMY].x - (x + 0.0)) / (MAXX+0.0);
	t.V[V_CENTER_ALL_ENEMY].y = (t.V[V_CENTER_ALL_ENEMY].y - (y + 0.0)) / (MAXY+0.0);

	t.V[V_CENTER_LIGHT_ENEMY].x = (t.V[V_CENTER_LIGHT_ENEMY].x - (x + 0.0)) / (MAXX+0.0);
	t.V[V_CENTER_LIGHT_ENEMY].y = (t.V[V_CENTER_LIGHT_ENEMY].y - (y + 0.0)) / (MAXY+0.0);

	t.V[V_CENTER_HEAVY_ENEMY].x = (t.V[V_CENTER_HEAVY_ENEMY].x - (x + 0.0)) / (MAXX+0.0);
	t.V[V_CENTER_HEAVY_ENEMY].y = (t.V[V_CENTER_HEAVY_ENEMY].y - (y + 0.0)) / (MAXY+0.0);

}



void post_iterate(Terminals& t) {
	if (t.R[R_RANGE_ANY_MATE_NUM] > t.R[R_RANGE_ANY_ENEMY_NUM]) {
		t.B[B_AM_BULLY] = true;
	} else {
		t.B[B_AM_BULLY] = false;
	}

	//danger is there is at least 5 enemies
	/*if (t.R[R_RANGE_ANY_ENEMY_NUM] > 5) {
		t.B[B_AM_ON_FIRE] = true;
	} else {
		t.B[B_AM_ON_FIRE] = false;
	}*/

	//a group if there is at least 4 mates
	if (t.R[R_RANGE_ANY_MATE_NUM] > 4) {
		t.B[B_AM_IN_GROUP] = true;
	} else {
		t.B[B_AM_IN_GROUP] = false;
	}

	if (t.hp_team > t.hp_enemy) {
		t.B[B_AM_WINNING] = true;
	} else {
		t.B[B_AM_WINNING] = false;
	}
		
}

void update_terminals2(const Game& game, Terminals& t, Memory& m) {

	// set on fire
	Unit* u = find_mate(t.me, m);
	if (u->hp > *(t.me->get_int_ptr("hp")) ) {
		t.B[B_AM_ON_FIRE] = true;
	} else {
		t.B[B_AM_ON_FIRE] = false;
	}

	//set just shoot
	if ( (u->last_tick_shoot != -1) && (u->last_tick_shoot > game.tick() - 3) ) {
		t.B[B_JUST_SHOT] = true;
	} else {
		t.B[B_JUST_SHOT] = false;
	}

	//set am colliding
	if ( (u->x == *(t.me->sod.x)) && (u->y == *(t.me->sod.y)) && ((u->x != u->x2) || (u->y != u->y2) )) {
		t.B[B_AM_COLLIDING] = true;
	} else {
		t.B[B_AM_COLLIDING] = false;
	}
	

	// set back
	if (t.B[B_AM_COLLIDING] == false) {
		t.V[V_ME_BACK].x = ( u->x - *(t.me->sod.x) ) / (MAXX+0.0);
		t.V[V_ME_BACK].y = ( u->y - *(t.me->sod.y) ) / (MAXX+0.0);
	} else {
		t.V[V_ME_BACK].x = ( u->x - u->x2 )  / (MAXX+0.0);
		t.V[V_ME_BACK].y = ( u->y - u->y2 ) / (MAXX+0.0);
	}
	

	// set path of the nearest mate
	
	Unit* mate = NULL;
	GameObj* gob = NULL;
	if (MARINE == t.code) {
		if (NULL != t.nearest_light_mate) {
			gob = t.nearest_light_mate;
			mate = find_mate(t.nearest_light_mate, m);
		}
	} else {
		if (NULL != t.nearest_heavy_mate) {
			gob = t.nearest_heavy_mate;
			mate = find_mate(t.nearest_heavy_mate, m);
		}
	}

	if (mate != NULL) {
		t.V[V_PATH_MATE].x = (*gob->sod.x - mate->x) / (MAXX+0.0);
		t.V[V_PATH_MATE].y = (*gob->sod.y - mate->y) / (MAXY+0.0);
	}
}



void update_terminals(const Game& game, GameObj* gob, Terminals& t) {
	set_me(gob, t);
	set_global(game, t);
	reset_vectors(t);
	iterate_over_mates(game, gob, t);
	iterate_over_enemies(game, gob, t);
	post_iterate(t);
}

Tuple& convert_vector_to_map(GameObj* gob, Terminals& t, Tuple& tp) {
	//std::cout << "GOTO1 : " << tp.x << " " << tp.y << endl;

	tp = NOR(tp);
	//std::cout << "GOTO2 : " << tp.x << " " << tp.y << endl;

	tp.x *= (t.max_speed * 3); //multiply times two to make a move more smooth
	tp.y *= (t.max_speed * 3); //maybe more  - 3
	//std::cout << "GOTO3 : " << tp.x << " " << tp.y << endl;

	tp.x = round(tp.x);
	tp.y = round(tp.y);
	//std::cout << "GOTO4 : " << tp.x << " " << tp.y << endl;

	tp.x += *gob->sod.x;
	tp.y += *gob->sod.y;
	//cout << "C "<< tp.x << " " << tp.y << endl;

	return tp;
}

